@@ -41,6 +41,31 @@
 #include "src/ie.h"
 #include "src/util.h"
 #include "src/handshake.h"
+#include "src/erp.h"
+#include "src/band.h"
+
+static inline unsigned int n_ecc_groups(void)
+{
+	const unsigned int *groups = l_ecc_supported_ike_groups();
+	unsigned int j = 0;
+
+	while (groups[j])
+		j += 1;
+
+	return j;
+}
+
+static inline int ecc_group_index(unsigned int group)
+{
+	const unsigned int *groups = l_ecc_supported_ike_groups();
+	int j;
+
+	for (j = 0; groups[j]; j++)
+		if (groups[j] == group)
+			return j;
+
+	return -ENOENT;
+}
 
 static bool handshake_get_nonce(uint8_t nonce[])
 {
@@ -51,6 +76,7 @@ static handshake_get_nonce_func_t get_nonce = handshake_get_nonce;
 static handshake_install_tk_func_t install_tk = NULL;
 static handshake_install_gtk_func_t install_gtk = NULL;
 static handshake_install_igtk_func_t install_igtk = NULL;
+static handshake_install_ext_tk_func_t install_ext_tk = NULL;
 
 void __handshake_set_get_nonce_func(handshake_get_nonce_func_t func)
 {
@@ -72,20 +98,49 @@ void __handshake_set_install_igtk_func(handshake_install_igtk_func_t func)
 	install_igtk = func;
 }
 
+void __handshake_set_install_ext_tk_func(handshake_install_ext_tk_func_t func)
+{
+	install_ext_tk = func;
+}
+
 void handshake_state_free(struct handshake_state *s)
 {
 	__typeof__(s->free) destroy = s->free;
 
+	if (s->in_event) {
+		s->in_event = false;
+		return;
+	}
+
 	l_free(s->authenticator_ie);
 	l_free(s->supplicant_ie);
+	l_free(s->authenticator_rsnxe);
+	l_free(s->supplicant_rsnxe);
 	l_free(s->mde);
 	l_free(s->fte);
+	l_free(s->fils_ip_req_ie);
+	l_free(s->fils_ip_resp_ie);
+	l_free(s->vendor_ies);
+
+	if (s->erp_cache)
+		erp_cache_put(s->erp_cache);
+
+	l_free(s->chandef);
 
 	if (s->passphrase) {
 		explicit_bzero(s->passphrase, strlen(s->passphrase));
 		l_free(s->passphrase);
 	}
 
+	if (s->ecc_sae_pts) {
+		unsigned int i;
+
+		for (i = 0; i < n_ecc_groups(); i++)
+			l_ecc_point_free(s->ecc_sae_pts[i]);
+
+		l_free(s->ecc_sae_pts);
+	}
+
 	explicit_bzero(s, sizeof(*s));
 
 	if (destroy)
@@ -130,82 +185,110 @@ void handshake_state_set_8021x_config(struct handshake_state *s,
 	s->settings_8021x = settings;
 }
 
-static bool handshake_state_setup_own_ciphers(struct handshake_state *s,
-						const struct ie_rsn_info *info)
-{
-	if (__builtin_popcount(info->pairwise_ciphers) != 1)
-		return false;
-
-	if (__builtin_popcount(info->akm_suites) != 1)
-		return false;
-
-	s->akm_suite = info->akm_suites;
-	s->pairwise_cipher = info->pairwise_ciphers;
-	s->group_cipher = info->group_cipher;
-	s->group_management_cipher = info->group_management_cipher;
-
-	/*
-	 * Don't set MFP for OSEN otherwise EAPoL will attempt to negotiate a
-	 * iGTK which is not allowed for OSEN.
-	 */
-	if (!s->osen_ie)
-		s->mfp = info->mfpc;
-
-	return true;
-}
-
 bool handshake_state_set_authenticator_ie(struct handshake_state *s,
 						const uint8_t *ie)
 {
 	struct ie_rsn_info info;
 
+	if (!ie_parse_rsne_from_data(ie, ie[1] + 2, &info))
+		goto valid_ie;
+
+	if (!ie_parse_wpa_from_data(ie, ie[1] + 2, &info))
+		goto valid_ie;
+
+	if (ie_parse_osen_from_data(ie, ie[1] + 2, &info) < 0)
+		return false;
+
+valid_ie:
 	l_free(s->authenticator_ie);
 	s->authenticator_ie = l_memdup(ie, ie[1] + 2u);
-	s->wpa_ie = is_ie_wpa_ie(ie + 2, ie[1]);
-	s->osen_ie = is_ie_wfa_ie(ie + 2, ie[1], IE_WFA_OI_OSEN);
 
-	if (!s->authenticator)
-		return true;
+	s->authenticator_ocvc = info.ocvc;
 
-	if (s->wpa_ie) {
-		if (ie_parse_wpa_from_data(ie, ie[1] + 2, &info) < 0)
-			return false;
-	} else if (s->osen_ie) {
-		if (ie_parse_osen_from_data(ie, ie[1] + 2, &info) < 0)
-			return false;
-	} else {
-		if (ie_parse_rsne_from_data(ie, ie[1] + 2, &info) < 0)
-			return false;
-	}
-
-	return handshake_state_setup_own_ciphers(s, &info);
+	return true;
 }
 
 bool handshake_state_set_supplicant_ie(struct handshake_state *s,
 						const uint8_t *ie)
 {
 	struct ie_rsn_info info;
+	bool wpa_ie = false;
+	bool osen_ie = false;
+
+	if (!ie_parse_rsne_from_data(ie, ie[1] + 2, &info))
+		goto valid_ie;
+
+	if (!ie_parse_wpa_from_data(ie, ie[1] + 2, &info)) {
+		wpa_ie = true;
+		goto valid_ie;
+	}
+
+	if (ie_parse_osen_from_data(ie, ie[1] + 2, &info) < 0)
+		return false;
+
+	osen_ie = true;
+
+valid_ie:
+	if (__builtin_popcount(info.pairwise_ciphers) != 1)
+		return false;
+
+	if (__builtin_popcount(info.akm_suites) != 1)
+		return false;
 
 	l_free(s->supplicant_ie);
 	s->supplicant_ie = l_memdup(ie, ie[1] + 2u);
-	s->wpa_ie = is_ie_wpa_ie(ie + 2, ie[1]);
-	s->osen_ie = is_ie_wfa_ie(ie + 2, ie[1], IE_WFA_OI_OSEN);
 
-	if (s->authenticator)
-		return true;
+	s->osen_ie = osen_ie;
+	s->wpa_ie = wpa_ie;
 
-	if (s->wpa_ie) {
-		if (ie_parse_wpa_from_data(ie, ie[1] + 2, &info) < 0)
-			return false;
-	} else if (s->osen_ie) {
-		if (ie_parse_osen_from_data(ie, ie[1] + 2, &info) < 0)
-			return false;
-	} else {
-		if (ie_parse_rsne_from_data(ie, ie[1] + 2, &info) < 0)
-			return false;
+	s->pairwise_cipher = info.pairwise_ciphers;
+	s->group_cipher = info.group_cipher;
+	s->group_management_cipher = info.group_management_cipher;
+	s->akm_suite = info.akm_suites;
+	s->supplicant_ocvc = info.ocvc;
+	s->ext_key_id_capable = info.extended_key_id;
+
+	/*
+	 * Don't set MFP for OSEN otherwise EAPoL will attempt to negotiate a
+	 * iGTK which is not allowed for OSEN.
+	 */
+	if (!s->osen_ie)
+		s->mfp = info.mfpc;
+
+	return true;
+}
+
+static void replace_ie(uint8_t **old, const uint8_t *new)
+{
+	if (*old == NULL) {
+		*old = new ? l_memdup(new, new[1] + 2) : NULL;
+		return;
 	}
 
-	return handshake_state_setup_own_ciphers(s, &info);
+	if (!new) {
+		l_free(*old);
+		*old = NULL;
+		return;
+	}
+
+	if ((*old)[1] == new[1] && !memcmp(*old, new, new[1] + 2))
+		return;
+
+	l_free(*old);
+	*old = l_memdup(new, new[1] + 2);
+}
+
+void handshake_state_set_authenticator_rsnxe(struct handshake_state *s,
+						const uint8_t *ie)
+{
+	l_free(s->authenticator_rsnxe);
+	s->authenticator_rsnxe = ie ? l_memdup(ie, ie[1] + 2) : NULL;
+}
+
+void handshake_state_set_supplicant_rsnxe(struct handshake_state *s,
+						const uint8_t *ie)
+{
+	replace_ie(&s->supplicant_rsnxe, ie);
 }
 
 void handshake_state_set_ssid(struct handshake_state *s, const uint8_t *ssid,
@@ -217,18 +300,39 @@ void handshake_state_set_ssid(struct handshake_state *s, const uint8_t *ssid,
 
 void handshake_state_set_mde(struct handshake_state *s, const uint8_t *mde)
 {
-	if (s->mde)
-		l_free(s->mde);
-
-	s->mde = mde ? l_memdup(mde, mde[1] + 2) : NULL;
+	replace_ie(&s->mde, mde);
 }
 
 void handshake_state_set_fte(struct handshake_state *s, const uint8_t *fte)
 {
-	if (s->fte)
-		l_free(s->fte);
+	replace_ie(&s->fte, fte);
+}
 
-	s->fte = fte ? l_memdup(fte, fte[1] + 2) : NULL;
+void handshake_state_set_vendor_ies(struct handshake_state *s,
+					const struct iovec *iov,
+					size_t n_iovs)
+{
+	size_t i;
+	size_t len;
+
+	l_free(s->vendor_ies);
+	s->vendor_ies = NULL;
+
+	if (n_iovs == 0) {
+		s->vendor_ies_len = 0;
+		return;
+	}
+
+	for (i = 0, len = 0; i < n_iovs; i++)
+		len += iov[i].iov_len;
+
+	s->vendor_ies_len = len;
+	s->vendor_ies = l_malloc(len);
+
+	for (i = 0, len = 0; i < n_iovs; i++) {
+		memcpy(s->vendor_ies + len, iov[i].iov_base, iov[i].iov_len);
+		len += iov[i].iov_len;
+	}
 }
 
 void handshake_state_set_kh_ids(struct handshake_state *s,
@@ -568,12 +672,29 @@ void handshake_state_install_ptk(struct handshake_state *s)
 
 		handshake_event(s, HANDSHAKE_EVENT_SETTING_KEYS);
 
-		install_tk(s, handshake_get_tk(s), cipher);
+		install_tk(s, s->active_tk_index, handshake_get_tk(s), cipher);
 	}
 }
 
+void handshake_state_install_ext_ptk(struct handshake_state *s,
+				uint8_t key_idx,
+				struct eapol_frame *ek, uint16_t proto,
+				bool noencrypt)
+{
+	s->ptk_complete = true;
+
+	if (install_ext_tk) {
+		uint32_t cipher =
+			ie_rsn_cipher_suite_to_cipher(s->pairwise_cipher);
+
+		install_ext_tk(s, key_idx, handshake_get_tk(s), cipher, ek,
+				proto, noencrypt);
+	}
+}
+
+
 void handshake_state_install_gtk(struct handshake_state *s,
-					uint8_t gtk_key_index,
+					uint16_t gtk_key_index,
 					const uint8_t *gtk, size_t gtk_len,
 					const uint8_t *rsc, uint8_t rsc_len)
 {
@@ -587,7 +708,7 @@ void handshake_state_install_gtk(struct handshake_state *s,
 }
 
 void handshake_state_install_igtk(struct handshake_state *s,
-					uint8_t igtk_key_index,
+					uint16_t igtk_key_index,
 					const uint8_t *igtk, size_t igtk_len,
 					const uint8_t *ipn)
 {
@@ -664,95 +785,78 @@ void handshake_state_set_gtk(struct handshake_state *s, const uint8_t *key,
  * results vs the RSN/WPA IE obtained as part of the 4-way handshake.  If they
  * don't match, the EAPoL packet must be silently discarded.
  */
-bool handshake_util_ap_ie_matches(const uint8_t *msg_ie,
+bool handshake_util_ap_ie_matches(const struct ie_rsn_info *msg_info,
 					const uint8_t *scan_ie, bool is_wpa)
 {
-	struct ie_rsn_info msg_info;
 	struct ie_rsn_info scan_info;
+	int r;
 
-	/*
-	 * First check that the sizes match, if they do, run a bitwise
-	 * comparison.
-	 */
-	if (msg_ie[1] == scan_ie[1] &&
-			!memcmp(msg_ie + 2, scan_ie + 2, msg_ie[1]))
-		return true;
+	if (!is_wpa)
+		r = ie_parse_rsne_from_data(scan_ie,
+						scan_ie[1] + 2, &scan_info);
+	else
+		r = ie_parse_wpa_from_data(scan_ie, scan_ie[1] + 2, &scan_info);
 
-	/*
-	 * Otherwise we have to parse the IEs and compare the individual
-	 * fields
-	 */
-	if (!is_wpa) {
-		if (ie_parse_rsne_from_data(msg_ie, msg_ie[1] + 2,
-						&msg_info) < 0)
-			return false;
-
-		if (ie_parse_rsne_from_data(scan_ie, scan_ie[1] + 2,
-						&scan_info) < 0)
-			return false;
-	} else {
-		if (ie_parse_wpa_from_data(msg_ie, msg_ie[1] + 2,
-						&msg_info) < 0)
-			return false;
-
-		if (ie_parse_wpa_from_data(scan_ie, scan_ie[1] + 2,
-						&scan_info) < 0)
-			return false;
-	}
-
-	if (msg_info.group_cipher != scan_info.group_cipher)
+	if (r < 0)
 		return false;
 
-	if (msg_info.pairwise_ciphers != scan_info.pairwise_ciphers)
+	if (msg_info->group_cipher != scan_info.group_cipher)
 		return false;
 
-	if (msg_info.akm_suites != scan_info.akm_suites)
+	if (msg_info->pairwise_ciphers != scan_info.pairwise_ciphers)
 		return false;
 
-	if (msg_info.preauthentication != scan_info.preauthentication)
+	if (msg_info->akm_suites != scan_info.akm_suites)
 		return false;
 
-	if (msg_info.no_pairwise != scan_info.no_pairwise)
+	if (msg_info->preauthentication != scan_info.preauthentication)
 		return false;
 
-	if (msg_info.ptksa_replay_counter != scan_info.ptksa_replay_counter)
+	if (msg_info->no_pairwise != scan_info.no_pairwise)
 		return false;
 
-	if (msg_info.gtksa_replay_counter != scan_info.gtksa_replay_counter)
+	if (msg_info->ptksa_replay_counter != scan_info.ptksa_replay_counter)
 		return false;
 
-	if (msg_info.mfpr != scan_info.mfpr)
+	if (msg_info->gtksa_replay_counter != scan_info.gtksa_replay_counter)
 		return false;
 
-	if (msg_info.mfpc != scan_info.mfpc)
+	if (msg_info->mfpr != scan_info.mfpr)
 		return false;
 
-	if (msg_info.peerkey_enabled != scan_info.peerkey_enabled)
+	if (msg_info->mfpc != scan_info.mfpc)
 		return false;
 
-	if (msg_info.spp_a_msdu_capable != scan_info.spp_a_msdu_capable)
+	if (msg_info->peerkey_enabled != scan_info.peerkey_enabled)
 		return false;
 
-	if (msg_info.spp_a_msdu_required != scan_info.spp_a_msdu_required)
+	if (msg_info->spp_a_msdu_capable != scan_info.spp_a_msdu_capable)
 		return false;
 
-	if (msg_info.pbac != scan_info.pbac)
+	if (msg_info->spp_a_msdu_required != scan_info.spp_a_msdu_required)
 		return false;
 
-	if (msg_info.extended_key_id != scan_info.extended_key_id)
+	if (msg_info->pbac != scan_info.pbac)
+		return false;
+
+	if (msg_info->extended_key_id != scan_info.extended_key_id)
+		return false;
+
+	if (msg_info->ocvc != scan_info.ocvc)
 		return false;
 
 	/* We don't check the PMKIDs since these might actually be different */
 
-	if (msg_info.group_management_cipher !=
+	if (msg_info->group_management_cipher !=
 			scan_info.group_management_cipher)
 		return false;
 
 	return true;
 }
 
-static const uint8_t *find_kde(const uint8_t *data, size_t data_len,
-				size_t *out_len, enum handshake_kde selector)
+const uint8_t *handshake_util_find_kde(enum handshake_kde selector,
+				const uint8_t *data, size_t data_len,
+				size_t *out_kde_len)
 {
 	struct ie_tlv_iter iter;
 	const uint8_t *result;
@@ -773,8 +877,8 @@ static const uint8_t *find_kde(const uint8_t *data, size_t data_len,
 		if (l_get_be32(result) != selector)
 			continue;
 
-		if (out_len)
-			*out_len = len - 4;
+		if (out_kde_len)
+			*out_kde_len = len - 4;
 
 		return result + 4;
 	}
@@ -786,8 +890,8 @@ const uint8_t *handshake_util_find_gtk_kde(const uint8_t *data, size_t data_len,
 						size_t *out_gtk_len)
 {
 	size_t gtk_len;
-	const uint8_t *gtk = find_kde(data, data_len, &gtk_len,
-					HANDSHAKE_KDE_GTK);
+	const uint8_t *gtk = handshake_util_find_kde(HANDSHAKE_KDE_GTK,
+						data, data_len, &gtk_len);
 
 	if (!gtk)
 		return NULL;
@@ -813,8 +917,8 @@ const uint8_t *handshake_util_find_igtk_kde(const uint8_t *data,
 						size_t *out_igtk_len)
 {
 	size_t igtk_len;
-	const uint8_t *igtk = find_kde(data, data_len, &igtk_len,
-					HANDSHAKE_KDE_IGTK);
+	const uint8_t *igtk = handshake_util_find_kde(HANDSHAKE_KDE_IGTK,
+						data, data_len, &igtk_len);
 
 	if (!igtk)
 		return NULL;
@@ -841,7 +945,8 @@ const uint8_t *handshake_util_find_pmkid_kde(const uint8_t *data,
 	const uint8_t *pmkid;
 	size_t pmkid_len;
 
-	pmkid = find_kde(data, data_len, &pmkid_len, HANDSHAKE_KDE_PMKID);
+	pmkid = handshake_util_find_kde(HANDSHAKE_KDE_PMKID, data, data_len,
+					&pmkid_len);
 
 	if (pmkid && pmkid_len != 16)
 		return NULL;
@@ -920,3 +1025,79 @@ bool handshake_decode_fte_key(struct handshake_state *s, const uint8_t *wrapped,
 
 	return true;
 }
+
+/* Add SAE-PT for ECC groups.  The group is carried by the point itself */
+bool handshake_state_add_ecc_sae_pt(struct handshake_state *s,
+						const struct l_ecc_point *pt)
+{
+	const struct l_ecc_curve *curve;
+	int i;
+
+	if (!pt)
+		return false;
+
+	curve = l_ecc_point_get_curve(pt);
+
+	if (!s->ecc_sae_pts)
+		s->ecc_sae_pts = l_new(struct l_ecc_point *, n_ecc_groups());
+
+	if ((i = ecc_group_index(l_ecc_curve_get_ike_group(curve))) < 0)
+		return false;
+
+	if (s->ecc_sae_pts[i])
+		return false;
+
+	s->ecc_sae_pts[i] = l_ecc_point_clone(pt);
+	return true;
+}
+
+void handshake_state_set_chandef(struct handshake_state *s,
+						struct band_chandef *chandef)
+{
+	if (s->chandef)
+		l_free(s->chandef);
+
+	s->chandef = chandef;
+}
+
+int handshake_state_verify_oci(struct handshake_state *s, const uint8_t *oci,
+				size_t oci_len)
+{
+	int r = -ENOENT;
+	bool ocvc;
+
+	l_debug("oci_len: %zu", oci ? oci_len : 0);
+
+	if (!oci)
+		goto done;
+
+	r = -EBADMSG;
+	if (oci_len != 3)
+		goto done;
+
+	l_debug("operating_class: %hu", oci[0]);
+	l_debug("primary_channel_number: %hu", oci[1]);
+	l_debug("frequency segment 1 channel number: %hu", oci[2]);
+
+	r = -EINVAL;
+
+	if (!s->chandef) {
+		l_debug("Own chandef unavailable");
+		goto done;
+	}
+
+	r = oci_verify(oci, s->chandef);
+	if (r < 0)
+		l_debug("OCI verification failed: %s", strerror(-r));
+
+done:
+	if (!r)
+		return r;
+
+	/* Only enforce validation if we're configured to do so */
+	ocvc = s->authenticator ? s->authenticator_ocvc : s->supplicant_ocvc;
+	if (!ocvc)
+		r = 0;
+
+	return r;
+}
