@@ -24,6 +24,9 @@
 #include <config.h>
 #endif
 
+#define _GNU_SOURCE
+#include <stdlib.h>
+
 #include <ell/ell.h>
 
 #include "src/missing.h"
